"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Deferred_1 = require("./util/Deferred");
class StatusUpdater {
    constructor() {
        this.deferred = new Deferred_1.Deferred();
        this.locked = false;
    }
    wait(task) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.locked) {
                let result = yield this.deferred;
                return result;
            }
            this.locked = true;
            try {
                let result = yield task();
                this.deferred.resolve(result);
                return result;
            }
            catch (e) {
                this.deferred.reject(e);
                throw new Error(e);
            }
            finally {
                this.resetLock();
            }
        });
    }
    resetLock() {
        setTimeout(() => {
            this.deferred = new Deferred_1.Deferred();
            this.locked = false;
        }, 1000);
    }
}
exports.StatusUpdater = StatusUpdater;
//# sourceMappingURL=StatusUpdater.js.map