"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const HAP_1 = require("../HAP");
class BatteryService {
    constructor(lock, accessory) {
        this.lock = lock;
        this.accessory = accessory;
        this.setupCharacteristics();
    }
    setupCharacteristics() {
        let batteryService = this.accessory.getService(HAP_1.HAP.Service.BatteryService);
        batteryService
            .getCharacteristic(HAP_1.HAP.Characteristic.BatteryLevel)
            .on('get', this.getBatteryLevel.bind(this));
        batteryService
            .getCharacteristic(HAP_1.HAP.Characteristic.ChargingState)
            .on('get', this.getBatteryChargingState.bind(this));
        batteryService
            .getCharacteristic(HAP_1.HAP.Characteristic.StatusLowBattery)
            .on('get', this.getLowBatteryStatus.bind(this));
    }
    getBatteryLevel(callback) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let status = yield this.lock.getStatus();
                callback(null, status.battery);
            }
            catch (e) {
                callback(e);
            }
        });
    }
    getLowBatteryStatus(callback) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let status = yield this.lock.getStatus();
                if (status.battery <= 20) {
                    callback(null, HAP_1.HAP.Characteristic.StatusLowBattery.BATTERY_LEVEL_LOW);
                }
                else {
                    callback(null, HAP_1.HAP.Characteristic.StatusLowBattery.BATTERY_LEVEL_NORMAL);
                }
            }
            catch (e) {
                callback(e);
            }
        });
    }
    getBatteryChargingState(callback) {
        callback(null, HAP_1.HAP.Characteristic.ChargingState.NOT_CHARGING);
    }
}
exports.BatteryService = BatteryService;
//# sourceMappingURL=BatteryService.js.map