"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const HAP_1 = require("../HAP");
const Logger_1 = require("../Logger");
const Client_1 = require("../Client");
class LockMechanismService {
    constructor(lock) {
        this.client = new Client_1.Client();
        this.lock = lock;
    }
    register(accessory) {
        this.accessory = accessory;
        let lockMechanismService = this.accessory.addService(HAP_1.HAP.Service.LockMechanism, this.lock.name);
        lockMechanismService
            .getCharacteristic(HAP_1.HAP.Characteristic.LockCurrentState)
            .on('get', this.getLockState.bind(this));
        lockMechanismService
            .getCharacteristic(HAP_1.HAP.Characteristic.LockTargetState)
            .on('get', this.getLockState.bind(this))
            .on('set', this.setLockState.bind(this));
    }
    getLockState(callback) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let status = yield this.client.getStatus(this.lock.id);
                this.lock.setStatus(status);
                if (status.locked) {
                    callback(null, HAP_1.HAP.Characteristic.LockCurrentState.SECURED);
                }
                else {
                    callback(null, HAP_1.HAP.Characteristic.LockCurrentState.UNSECURED);
                }
            }
            catch (e) {
                Logger_1.Logger.error('Unable to get lock state', e);
                callback(e);
            }
        });
    }
    setLockState(targetState, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            let lockMechanismService = this.accessory.getService(HAP_1.HAP.Service.LockMechanism);
            Logger_1.Logger.log(`${this.lock.name} is ${targetState ? 'locking' : 'unlocking'}...`);
            try {
                yield this.client.control(this.lock.id, targetState);
                Logger_1.Logger.log(`${this.lock.name} is ${targetState ? 'locked' : 'unlocked'}`);
                lockMechanismService.getCharacteristic(HAP_1.HAP.Characteristic.LockCurrentState)
                    .updateValue(targetState);
                callback(null);
            }
            catch (e) {
                Logger_1.Logger.error('Unable to set lock state', e);
                callback(e);
            }
        });
    }
}
exports.LockMechanismService = LockMechanismService;
//# sourceMappingURL=LockMechanismService.js.map