"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const HAP_1 = require("../HAP");
const Logger_1 = require("../Logger");
class LockService {
    constructor(lock, accessory) {
        this.lock = lock;
        this.accessory = accessory;
        this.setupCharacteristics();
    }
    setupCharacteristics() {
        let lockMechanismService = this.getOrCreateLockMechanismService();
        lockMechanismService
            .getCharacteristic(HAP_1.HAP.Characteristic.LockCurrentState)
            .on('get', this.getCurrentLockState.bind(this));
        lockMechanismService
            .getCharacteristic(HAP_1.HAP.Characteristic.LockTargetState)
            .on('get', this.getTargetLockState.bind(this))
            .on('set', this.setLockState.bind(this));
    }
    getOrCreateLockMechanismService() {
        let lockMechanismService = this.accessory.getService(HAP_1.HAP.Service.LockMechanism);
        if (!lockMechanismService) {
            lockMechanismService = this.accessory.addService(HAP_1.HAP.Service.LockMechanism, this.lock.name);
        }
        return lockMechanismService;
    }
    getCurrentLockState(callback) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let status = yield this.lock.getStatus();
                if (!status.responsive) {
                    Logger_1.Logger.log(`${this.lock.name} is unresponsive, forcing a status sync...`);
                    yield this.lock.syncStatus();
                    status = yield this.lock.getStatus();
                    if (!status.responsive) {
                        throw new Error(`${this.lock.status} is still unresponsive. Check WiFi connectivity.`);
                    }
                }
                if (status.locked) {
                    callback(null, HAP_1.HAP.Characteristic.LockCurrentState.SECURED);
                }
                else {
                    callback(null, HAP_1.HAP.Characteristic.LockCurrentState.UNSECURED);
                }
            }
            catch (e) {
                Logger_1.Logger.error('Unable to get lock state', e);
                callback(e);
            }
        });
    }
    getTargetLockState(callback) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let status = yield this.lock.getStatus();
                if (status.locked) {
                    callback(null, HAP_1.HAP.Characteristic.LockCurrentState.SECURED);
                }
                else {
                    callback(null, HAP_1.HAP.Characteristic.LockCurrentState.UNSECURED);
                }
            }
            catch (e) {
                Logger_1.Logger.error('Unable to get lock state', e);
                callback(e);
            }
        });
    }
    setLockState(targetState, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            let lockMechanismService = this.getOrCreateLockMechanismService();
            Logger_1.Logger.log(`${this.lock.name} is ${targetState ? 'locking' : 'unlocking'}...`);
            try {
                yield this.lock.setLockedState(targetState);
                Logger_1.Logger.log(`${this.lock.name} is ${targetState ? 'locked' : 'unlocked'}`);
                lockMechanismService.getCharacteristic(HAP_1.HAP.Characteristic.LockCurrentState)
                    .updateValue(targetState);
                callback();
            }
            catch (e) {
                Logger_1.Logger.error('Unable to set lock state', e);
                callback(e);
            }
        });
    }
}
exports.LockService = LockService;
//# sourceMappingURL=LockService.js.map