"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const request = require("request-promise");
const store = require("store");
const Logger_1 = require("./Logger");
class Client {
    constructor() {
        this.API_URI = 'https://api.candyhouse.co/public';
        this.MAX_RETRIES = 20;
        this.POLL_DELAY = 1000;
        this.token = store.get('token');
    }
    listLocks() {
        return __awaiter(this, void 0, void 0, function* () {
            let options = this.buildRequestOptions('sesames');
            let response = yield request.get(options);
            Logger_1.Logger.debug('Got listLocks response', response);
            return response;
        });
    }
    getStatus(id) {
        return __awaiter(this, void 0, void 0, function* () {
            let options = this.buildRequestOptions(`sesame/${id}`);
            let status = yield request.get(options);
            Logger_1.Logger.debug('Got status response', status);
            return status;
        });
    }
    control(id, secure) {
        return __awaiter(this, void 0, void 0, function* () {
            let options = this.buildRequestOptions(`sesame/${id}`);
            options.body = { command: (secure ? 'lock' : 'unlock') };
            let response = yield request.post(options);
            let result = yield this.waitForTask(response.task_id);
            return result;
        });
    }
    sync(id) {
        return __awaiter(this, void 0, void 0, function* () {
            let options = this.buildRequestOptions(`sesame/${id}`);
            options.body = { command: 'sync' };
            let response = yield request.post(options);
            let result = yield this.waitForTask(response.task_id);
            return result;
        });
    }
    getTaskStatus(task_id) {
        return __awaiter(this, void 0, void 0, function* () {
            let options = this.buildRequestOptions('action-result');
            options.qs = { task_id: task_id };
            let status = yield request.get(options);
            return status;
        });
    }
    waitForTask(task_id) {
        return __awaiter(this, void 0, void 0, function* () {
            let retries = this.MAX_RETRIES;
            let result;
            while (retries-- > 0) {
                if (retries == 0) {
                    throw new Error('Control task took too long to complete.');
                }
                Logger_1.Logger.debug(`Waiting for task to complete. Attempts remaining: ${retries}/${this.MAX_RETRIES}`);
                yield this.delay(this.POLL_DELAY);
                result = yield this.getTaskStatus(task_id);
                Logger_1.Logger.debug('Task response', result);
                if (result.status == 'processing') {
                    continue;
                }
                return result;
            }
        });
    }
    buildRequestOptions(path) {
        let options = {
            uri: `${this.API_URI}/${path}`,
            json: true,
            headers: {
                'Content-Type': 'application/json',
                'Authorization': this.token
            }
        };
        return options;
    }
    delay(ms) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise(resolve => setTimeout(resolve, ms));
        });
    }
}
exports.Client = Client;
//# sourceMappingURL=Client.js.map