"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const store = require("store");
const Client_1 = require("./Client");
const HAP_1 = require("./HAP");
const LockAccessory_1 = require("./LockAccessory");
const Logger_1 = require("./Logger");
const Server_1 = require("./Server");
class LockPlatform {
    constructor(log, config, platform) {
        Logger_1.Logger.setLogger(log, config.debug);
        this.platform = platform;
        this.accessories = new Map();
        this.server = new Server_1.Server(config.port);
        this.platform.on('didFinishLaunching', () => {
            if (!config.token) {
                throw new Error('A token was not found in the homebridge config. For more information, see: https://github.com/aschzero/homebridge-sesame#configuration');
            }
            store.set('token', config.token);
            this.retrieveLocks();
            this.server.listen();
        });
    }
    retrieveLocks() {
        return __awaiter(this, void 0, void 0, function* () {
            let client = new Client_1.Client();
            let locks;
            try {
                locks = yield client.listLocks();
            }
            catch (e) {
                Logger_1.Logger.error('Unable to retrieve locks', e);
            }
            locks.forEach(lock => this.addAccessory(lock));
        });
    }
    configureAccessory(accessory) {
        this.accessories.set(accessory.UUID, accessory);
    }
    addAccessory(lock) {
        let uuid = HAP_1.HAP.UUID.generate(lock.device_id);
        let accessory = this.accessories.get(uuid);
        if (!accessory) {
            accessory = new HAP_1.HAP.Accessory(lock.nickname, uuid);
            this.platform.registerPlatformAccessories('homebridge-sesame', 'Sesame', [accessory]);
        }
        let lockAccessory = new LockAccessory_1.LockAccessory(lock, accessory);
        this.server.locks.set(lock.device_id, lockAccessory);
        Logger_1.Logger.log(`Found ${lock.nickname}`);
        return accessory;
    }
}
exports.LockPlatform = LockPlatform;
//# sourceMappingURL=LockPlatform.js.map