"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const express = require("express");
const Logger_1 = require("./Logger");
class Server {
    constructor(port) {
        this.DEFAULT_PORT = 33892;
        this.port = port || this.DEFAULT_PORT;
        this.locks = new Map();
        this.api = express();
        this.api.use(express.json());
        this.api.post('/', (req, res) => {
            try {
                this.handleRequest(req);
            }
            catch (e) {
                Logger_1.Logger.error(e);
            }
            res.end();
        });
    }
    listen() {
        this.api.listen(this.port, () => Logger_1.Logger.log(`Listening for webhooks on port ${this.port}`));
    }
    handleRequest(request) {
        let id = request.body.device_id;
        let locked = request.body.locked;
        if (id == null || locked == null) {
            Logger_1.Logger.log(`Unexpected webhook request body: ${JSON.stringify(request.body)}`);
            return;
        }
        let lockAccessory = this.locks.get(id);
        if (!lockAccessory) {
            Logger_1.Logger.log(`No lock accessory found from webhook request. Device ID: ${id}`);
            return;
        }
        lockAccessory.updateCurrentLockState(locked);
        Logger_1.Logger.log(`Set ${lockAccessory.lock.nickname} to ${locked ? 'locked' : 'unlocked'} from webhook`);
    }
}
exports.Server = Server;
//# sourceMappingURL=Server.js.map