"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const HAP_1 = require("../HAP");
const Logger_1 = require("../Logger");
class LockService {
    constructor(lock, service) {
        this.lock = lock;
        this.service = service;
        this.service
            .getCharacteristic(HAP_1.HAP.Characteristic.LockCurrentState)
            .on('get', this.getCurrentLockState.bind(this));
        this.service
            .getCharacteristic(HAP_1.HAP.Characteristic.LockTargetState)
            .on('get', this.getTargetLockState.bind(this))
            .on('set', this.setLockState.bind(this));
    }
    getCurrentLockState(callback) {
        return __awaiter(this, void 0, void 0, function* () {
            let status;
            try {
                status = yield this.lock.mutex.wait(() => this.lock.client.getStatus(this.lock.id));
            }
            catch (e) {
                Logger_1.Logger.error('Unable to get lock state', e);
                callback(e);
            }
            if (!status.responsive) {
                Logger_1.Logger.log(`${this.lock.name} is unresponsive, forcing a status sync...`);
                try {
                    let result = yield this.lock.client.sync(this.lock.id);
                    if (result.successful) {
                        Logger_1.Logger.log(`${this.lock.name} sync successful.`);
                    }
                    else {
                        Logger_1.Logger.error(`${this.lock.name} failed to sync, please check WiFi connectivity. API responded with: ${result.error}`);
                    }
                }
                catch (e) {
                    Logger_1.Logger.error(`Unable to sync`, e);
                }
            }
            if (status.locked) {
                callback(null, HAP_1.HAP.Characteristic.LockCurrentState.SECURED);
            }
            else {
                callback(null, HAP_1.HAP.Characteristic.LockCurrentState.UNSECURED);
            }
        });
    }
    getTargetLockState(callback) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let status = yield this.lock.mutex.wait(() => this.lock.client.getStatus(this.lock.id));
                if (status.locked) {
                    callback(null, HAP_1.HAP.Characteristic.LockCurrentState.SECURED);
                }
                else {
                    callback(null, HAP_1.HAP.Characteristic.LockCurrentState.UNSECURED);
                }
            }
            catch (e) {
                Logger_1.Logger.error('Unable to get lock state', e);
                callback(e);
            }
        });
    }
    setLockState(targetState, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.Logger.log(`${this.lock.name} is ${targetState ? 'locking' : 'unlocking'}...`);
            try {
                yield this.lock.client.control(this.lock.id, targetState);
            }
            catch (e) {
                Logger_1.Logger.error('Unable to set lock state', e);
                callback(e);
            }
            Logger_1.Logger.log(`${this.lock.name} is ${targetState ? 'locked' : 'unlocked'}`);
            this.service.getCharacteristic(HAP_1.HAP.Characteristic.LockCurrentState)
                .updateValue(targetState);
            callback();
        });
    }
}
exports.LockService = LockService;
//# sourceMappingURL=LockService.js.map